@@ -1,7 +1,7 @@
 /* glpkAPI.h
    R interface to GLPK.
  
-   Copyright (C) 2011-2012 Gabriel Gelius-Dietrich, Dpt. for Bioinformatics,
+   Copyright (C) 2011-2013 Gabriel Gelius-Dietrich, Dpt. for Bioinformatics,
    Institute for Informatics, Heinrich-Heine-University, Duesseldorf, Germany.
    All right reserved.
    Email: geliudie@uni-duesseldorf.de
@@ -222,7 +222,7 @@ SEXP getObjCoefs(SEXP lp, SEXP j);
 SEXP getObjCoef(SEXP lp, SEXP j);
 
 /* load the whole constraint matrix */
-SEXP loadMatrix(SEXP lp, SEXP ne, SEXP ia, SEXP ja, SEXP ra, SEXP check);
+SEXP loadMatrix(SEXP lp, SEXP ne, SEXP ia, SEXP ja, SEXP ra);
 
 /* check for duplicate elements in sparse matrix */
 SEXP checkDup(SEXP m, SEXP n, SEXP ne, SEXP ia, SEXP ja);
